@echo off
REM This batch file builds file.bin from file.asm
if not arg%1==arg%1 goto arg
echo supply base name of file.asm, i.e. file
goto exit
:arg
echo on
masm %1,,;
link boot-hdp,;
@echo Ignore the 'no stack...' warning
del %1.obj
exe2bin %1
del %1.exe
:exit
